@@ -1,5 +1,5 @@
 /// Help text.
-pub const HELP: &str = "# felix v2.3.0
+pub const HELP: &str = r###"# felix v2.4.0
 A simple TUI file manager with vim-like keymapping.
 
 ## Usage
@@ -15,19 +15,29 @@ Both relative and absolute path available.
 j / Down          :Go down.
 k / Up            :Go up.
 h / Left          :Go to the parent directory if exists.
-l / Right / Enter :Open a file or change directory.
+l / Right / Enter :Open item or change directory.
 gg                :Go to the top.
 G                 :Go to the bottom.
 z + Enter         :Go to the home directory.
 z <keyword>       :Jump to a directory that matches the keyword. (zoxide required)
-o                 :Open a file in a new window.
+o                 :Open item in a new window.
 e                 :Unpack archive/compressed file.
-dd                :Delete and yank one item.
-yy                :Yank one item.
+dd                :Delete and yank item.
+yy                :Yank item.
 p                 :Put yanked item(s) in the current directory.
-V                 :Switch to the select mode.
-  - d             :In the select mode, delete and yank selected item(s).
-  - y             :In the select mode, yank selected item(s).
+:reg              :Show registers. To hide it, press v.
+"ayy              :Yank item to register a.
+"add              :Delete and yank item to register a.
+"Ayy              :Append item to register a.
+"Add              :Delete and append item to register a.
+"ap               :Put item(s) from register a.
+V                 :Switch to the linewise visual mode.
+  - y             :In the visual mode, yank selected item(s).
+  - d             :In the visual mode, delete and yank selected item(s).
+  - "ay           :In the visual mode, yank items to register a.
+  - "ad           :In the visual mode, delete and yank items to register a.
+  - "Ay           :In the visual mode, append items to register a.
+  - "Ad           :In the visual mode, delete and append items to register a.
 u                 :Undo put/delete/rename.
 Ctrl + r          :Redo put/delete/rename.
 v                 :Toggle whether to show the preview.
@@ -38,12 +48,11 @@ backspace         :Toggle whether to show hidden items.
 t                 :Toggle the sort order (name <-> modified time).
 :                 :Switch to the shell mode.
 c                 :Switch to the rename mode.
-/                 :Search items by the keyword.
+/                 :Search items by a keyword.
 n                 :Go forward to the item that matches the keyword.
 N                 :Go backward to the item that matches the keyword.
 Esc               :Return to the normal mode.
-:cd / :z          :Go to the home directory.
-:z <keyword>      :Same as `z <keyword>`.
+:cd               :Go to the home directory.
 :e                :Reload the current directory.
 :trash            :Go to the trash directory.
 :empty            :Empty the trash directory.
@@ -62,7 +71,11 @@ trash directory: $XDG_DATA_HOME/felix/trash
 log files      : $XDG_DATA_HOME/felix/log
 
 ### macOS
-config file    : $HOME/Library/Application Support/felix/config.yaml
+On macOS, felix looks for the config file in the following locations:
+
+1. `$HOME/Library/Application Support/felix/config.yaml`
+2. `$HOME/.config/felix/config.yaml`
+
 trash directory: $HOME/Library/Application Support/felix/trash
 log files      : $HOME/Library/Application Support/felix/log
 
@@ -72,4 +85,4 @@ trash directory : $PROFILE\\AppData\\Local\\felix\\trash
 log files       : $PROFILE\\AppData\\Local\\felix\\log
 
 For more details, visit https://github.com/kyoheiu/felix
-";
+"###;
